local Colors={}
Recount.Colors=Colors

ItemsToUpdate={}
TypeToUpdate={}
ColorMultiplier={}
TextureBackgroundsToUpdate={}


local TYPE_TEXTURE=1
local TYPE_BORDER=2
local TYPE_BACKGROUND=3

local Cur_Branch
local Cur_Name
local TempColor={}

local function Color_Change()
	local r, g, b = ColorPickerFrame:GetColorRGB()
	
	TempColor.r=r
	TempColor.g=g
	TempColor.b=b
	if not ColorPickerFrame.hasOpacity then
		TempColor.a=nil
	else
		TempColor.a=1.0-OpacitySliderFrame:GetValue()
	end
	
	Colors:SetColor(Cur_Branch,Cur_Name,TempColor)
end

local function Opacity_Change()	
	local r, g, b = ColorPickerFrame:GetColorRGB()
	local a=1.0-OpacitySliderFrame:GetValue()

	TempColor.r=r
	TempColor.g=g
	TempColor.b=b
	TempColor.a=a	

	Colors:SetColor(Cur_Branch,Cur_Name,TempColor)
end

local function Fake_Change()
end

local function Color_Cancel()	
	Colors:SetColor(Cur_Branch,Cur_Name,ColorPickerFrame.previousValues)
end

function Colors:GetColor(Branch,Name)
	if Branch=="Class" then
		if not Recount.db.profile.Colors[Branch][Name] then
			return RAID_CLASS_COLORS[Name]
		end
	end
	return Recount.db.profile.Colors[Branch][Name]
end

function Colors:SetColor(Branch,Name,c)
	Recount.db.profile.Colors[Branch][Name]=c

	local Items=ItemsToUpdate[Branch][Name]

	for k, v in pairs(TypeToUpdate[Branch][Name]) do
		if v==TYPE_TEXTURE then			
			Multi=ColorMultiplier[Branch][Name][k]
			if c.a then		
				if Multi then
					Items[k]:SetVertexColor(c.r*Multi.r,c.g*Multi.g,c.b*Multi.b,c.a*Multi.a)
				else
					Items[k]:SetVertexColor(c.r,c.g,c.b,c.a)
				end
			else
				if Multi then
					Items[k]:SetVertexColor(c.r*Multi.r,c.g*Multi.g,c.b*Multi.b)
				else
					Items[k]:SetVertexColor(c.r,c.g,c.b)
				end
			end
		elseif v==TYPE_BORDER then
			if c.a then
				Items[k]:SetBackdropBorderColor(c.r,c.g,c.b,c.a)
			else
				Items[k]:SetBackdropBorderColor(c.r,c.g,c.b)
			end
		elseif v==TYPE_BACKGROUND then
			if c.a then
				Items[k]:SetBackdropColor(c.r,c.g,c.b,c.a)
			else
				Items[k]:SetBackdropColor(c.r,c.g,c.b)
			end
		end
	end
end


function Colors:UnregisterItem(Item)
	for k1,Branch in pairs(ItemsToUpdate) do
		for k2,Name in pairs(Branch) do
			for k,v in pairs(Name) do
				if v==Item then
					Name[k]=nil
					TypeToUpdate[k1][k2][k]=nil
					ColorMultiplier[k1][k2][k]=nil
				end
			end
		end
	end
end

function Colors:RegisterTexture(Branch,Name,Texture, Multi)
	local c=Colors:GetColor(Branch,Name)
	if c.a then
		if Multi then
			Texture:SetVertexColor(c.r*Multi.r,c.g*Multi.g,c.b*Multi.b,c.a*Multi.a)
		else
			Texture:SetVertexColor(c.r,c.g,c.b,c.a)
		end
	else
		if Multi then
			Texture:SetVertexColor(c.r*Multi.r,c.g*Multi.g,c.b*Multi.b)
		else
			Texture:SetVertexColor(c.r,c.g,c.b)
		end
	end

	if type(ItemsToUpdate[Branch])~="table" then
		ItemsToUpdate[Branch]={}
		TypeToUpdate[Branch]={}
		ColorMultiplier[Branch]={}
	end

	if type(ItemsToUpdate[Branch][Name])~="table" then
		ItemsToUpdate[Branch][Name]={}
		TypeToUpdate[Branch][Name]={}
		ColorMultiplier[Branch][Name]={}
	end

	local entry=#ItemsToUpdate[Branch][Name]+1
	table.insert(ItemsToUpdate[Branch][Name],Texture)
	table.insert(TypeToUpdate[Branch][Name],TYPE_TEXTURE)

	if Multi then
		ColorMultiplier[Branch][Name][entry]=Multi
	end
end

function Colors:RegisterBorder(Branch,Name,frame)
	local c=Colors:GetColor(Branch,Name)
	if c.a then
		frame:SetBackdropBorderColor(c.r,c.g,c.b,c.a)
	else
		frame:SetBackdropBorderColor(c.r,c.g,c.b)
	end

	if type(ItemsToUpdate[Branch])~="table" then
		ItemsToUpdate[Branch]={}
		TypeToUpdate[Branch]={}
		ColorMultiplier[Branch]={}
	end

	if type(ItemsToUpdate[Branch][Name])~="table" then
		ItemsToUpdate[Branch][Name]={}
		TypeToUpdate[Branch][Name]={}
		ColorMultiplier[Branch][Name]={}
	end

	table.insert(ItemsToUpdate[Branch][Name],frame)
	table.insert(TypeToUpdate[Branch][Name],TYPE_BORDER)
end

function Colors:RegisterBackground(Branch,Name,frame)
	local c=Colors:GetColor(Branch,Name)
	if c.a then
		frame:SetBackdropColor(c.r,c.g,c.b,c.a)
	else
		frame:SetBackdropColor(c.r,c.g,c.b)
	end

	if type(ItemsToUpdate[Branch])~="table" then
		ItemsToUpdate[Branch]={}
		TypeToUpdate[Branch]={}
		ColorMultiplier[Branch]={}
	end

	if type(ItemsToUpdate[Branch][Name])~="table" then
		ItemsToUpdate[Branch][Name]={}
		TypeToUpdate[Branch][Name]={}
		ColorMultiplier[Branch][Name]={}
	end

	table.insert(ItemsToUpdate[Branch][Name],frame)
	table.insert(TypeToUpdate[Branch][Name],TYPE_BACKGROUND)
end

--/script Recount.Colors:EditColor("Window","Title",Recount.MainWindow)
function Colors:EditColor(Branch,Name,Attach)
	Cur_Branch=Branch
	Cur_Name=Name

	PlaySound("igMainMenuOptionCheckBoxOn")
	local color=Colors:GetColor(Branch,Name)

	ColorPickerFrame.hasOpacity = false
	ColorPickerFrame.func=Fake_Change	
	ColorPickerFrame.opacityFunc=Fake_Change
	ColorPickerFrame:SetColorRGB(color.r, color.g, color.b)

	if color.a then
		ColorPickerFrame.opacity = 1.0 - color.a
		ColorPickerFrame.opacityFunc=Opacity_Change
		ColorPickerFrame.hasOpacity = true
	end
	ColorPickerFrame.previousValues = color
	ColorPickerFrame:ClearAllPoints()
	if Attach then
		ColorPickerFrame:SetPoint("RIGHT",Attach,"LEFT",0,0);
	end
				
	ColorPickerFrame.cancelFunc = Color_Cancel
	ColorPickerFrame.func=Color_Change	
	ColorPickerFrame.opacityFunc=Opacity_Change
	ColorPickerFrame:Show()

end